//
//	UModalFilter.cp
//
//	Generic support for Modal Filter Procedures in PowerPlant.
//	This is not a real class, but a holder for some static routines.
//

#include "UModalFilter.h"

ModalFilterUPP UModalFilter::sModalFilterUPP = nil;

EFilterBehaviour UModalFilter::sBehaviour = filter_AlertDefaultBehaviour;

ModalFilterUPP	UModalFilter::GetFilter( EFilterBehaviour inBehaviour )
{	// Gets the Modal Filter Procedure pointer

	// Note that we only allocate one UPP. We never need to destroy it.
	if ( sModalFilterUPP == nil )
		sModalFilterUPP = NewModalFilterProc( sFilter );
	
	// Put the requested behaviour where the filter can find it.
	sBehaviour = inBehaviour;
	
	return sModalFilterUPP;

}

// sFilter is closely derived from TechNote TB 37
// Comments that were in the original TechNote are in /* old style comments */
pascal Boolean UModalFilter::sFilter(
					DialogPtr 	 inCurrentDialog, 
					EventRecord *inEvent, 
					short 		*inDialogItem)
{
	Boolean returnVal = false;
	
	switch ( inEvent->what ) {
	
	case updateEvt:
		if ( (DialogPtr)inEvent->message == inCurrentDialog ) {
		
			// Handle update for this dialog: need to redraw the default button outline.
			// Dialog manager does the rest.
			if ( (sBehaviour & filter_DefaultIsButton1) != 0 ) {
				OutlineButton( inCurrentDialog, 1 );
			}
			
		} else {
		
			// Handle update for other windows
			// Do exactly what LEventDispatcher::EventUpdate() does.
			LWindow	*theWindow = LWindow::FetchWindowObject(
											(WindowPtr) inEvent->message);
			if (theWindow != nil) {
				theWindow->UpdatePort();
			}
			
		}
		break;
		
	case keyDown:
	case autoKey:
		if ( TestForButtonEquivalent( inCurrentDialog, inEvent, 1) ) {
			
			// simulate pressing button 1
			*inDialogItem = 1;
			SimulateButtonPress( inCurrentDialog, *inDialogItem );
			returnVal = true;
			break;
			
		} else if ( TestForButtonEquivalent( inCurrentDialog, inEvent, 2) ) {
			
			// simulate pressing button 2
			*inDialogItem = 2;
			SimulateButtonPress( inCurrentDialog, *inDialogItem );
			returnVal = true;
			break;
			
	    }
		goto DefaultPoint;	// other keys pass to system's filter
		
	case nullEvent:
		// idlers get time on every null event
		LPeriodical::DevoteTimeToIdlers( *inEvent );
		goto DefaultPoint;	// all nulls go to system's filter
		
	default:
DefaultPoint:
		if ( (sBehaviour & filter_System6Friendly) == 0 ) {
			// System 7 only: get and call the toolboxes StdFilterProc.
			/* it wasnt an update, pass it on to the system filter */
			StPortOriginState thePort( inCurrentDialog );
						/* save the current port */
						/* and set to the dialog, this is necessary */
						/* to track the edit line */
						/* cursor change correctly */	
			/* not using MPW 3.2, get and call the standard filter myself */
			/* get the standard system dialog filter address */
			ModalFilterUPP standardProc;
			OSErr myErr = ::GetStdFilterProc(&standardProc);
			if(!myErr)
				returnVal= CallModalFilterProc(standardProc, 
								inCurrentDialog,inEvent,inDialogItem);
			
		}
		break;
	}
	
	// Repeaters get time after every event
	LPeriodical::DevoteTimeToRepeaters( *inEvent );
	
	return returnVal;
}

Boolean UModalFilter::TestForButtonEquivalent( DialogPtr inDialog, EventRecord *inEvent, Int16 inButton)
{
	if ( !(inEvent->what == keyDown || inEvent->what == autoKey) ) return false;
	
	Char16 theChar = (inEvent->message & charCodeMask);
	
	if ( inButton == 1 ) {
	
		if ( !(theChar == char_Return || theChar==char_Enter) ) return false;
		if ( (sBehaviour & filter_DefaultIsButton1) == 0 ) return false;
		
	} else if ( inButton == 2 ) {
		
		// Note: Escape has the same char code as Clear, so we have to check the key code too.
		if ( !(	(inEvent->message & 0x00007FFF) == vkey_Escape + char_Escape
				|| UKeyFilters::IsCmdPeriod(*inEvent) ) ) return false;
		if ( (sBehaviour & filter_CancelIsButton2) == 0 ) return false;
		
	} else 
		return false;
	
#if !STRICT_WINDOWS
	// make sure the dialog has the item
	short maxItem = *((short*)*(((DialogPeek)inDialog)->items)) + 1;
	if ( inButton > maxItem ) return false;
#endif
	
	// find the item, make sure it's an enabled button
	short itemType;
	ControlHandle theControl;
	Rect box;
	::GetDialogItem( inDialog, inButton, &itemType, &(Handle)theControl, &box );
	
	if ( itemType != btnCtrl + ctrlItem ) return false;	// item not a button
	
	if ( (*theControl)->contrlVis == 0 ) return false;	// control invisible
	
	if ( (*theControl)->contrlHilite == 255 ) return false;// control inactive
	
	// we've passed all tests
	return true;
}

void	UModalFilter::SimulateButtonPress( DialogPtr inDialog, short inButton )
{
	long finalTicks;	// ignored
	const long kVisualDelay = 8;
	short itemType;
	ControlHandle theControl;
	Rect box;
	::GetDialogItem( inDialog, inButton, &itemType, &(Handle)theControl, &box );
	
	Assert_(itemType == btnCtrl + ctrlItem);
	
	::HiliteControl( theControl, 1 );
	::Delay( kVisualDelay, &finalTicks );
	::HiliteControl( theControl, 0 );
	
}

void	UModalFilter::OutlineButton( DialogPtr inDialog, short inButton )
{
	StPortOriginState thePort( inDialog );
	
	short itemType;
	ControlHandle theControl;
	Rect frame;
	::GetDialogItem( inDialog, inButton, &itemType, &(Handle)theControl, &frame );
	
	Assert_(itemType == btnCtrl + ctrlItem);
	
	::InsetRect( &frame, -4, -4 );
	Int16	roundCorner = (frame.bottom - frame.top - 8) / 2;
	
	if (roundCorner < 16) {
		roundCorner = 16;
	}
	
	// To do (?): Set fore and back colors based on button's activation state.
	// To do (?): Do an outline based on the Apple Grayscale Appearance.
	
	::PenNormal();
	::PenSize(3, 3);
	::FrameRoundRect(&frame, roundCorner, roundCorner);
	
}
