//
//	UModalFilter.h
//
//	Generic support for Modal Filter Procedures in PowerPlant.
//	This is not a "real" class, but a holder for some static routines.
//
//	Based on Tech Note TB 37
//	Requires System 7
//	Allows the use of:
//		err = ::SetDialogDefaultItem(theDialog,newItem);
//		err = ::SetDialogCancelItem(theDialog,newItem);
//		err = ::SetDialogTrackCursor(theDialog,tracks);
//	
//	Use it like this:
//		::Alert( alertID_MyAlert, UModalFilter::GetFilter() );
//	or
//		::ModalDialog( UModalFilter::GetFilter(0), &theItemHit );
//
//	The optional argument to GetFilter() tells the filter which of the SetDialog...()
//	calls have *not* been set. For example:
//		GetFilter( defaultIsButton1 | cancelIsButton2 );
//	asks the filter to handle return --> button 1, escape --> button 2, while
//		GetFilter( noBehaviour ) or GetFilter( 0 )
//	tells the filter to pass everything to the system's StdFilterProc, which is 
//	what you want if you have called the SetDialog...() functions.
//

#pragma once

enum EFilterBehaviour {
		filter_NoBehaviour			= 0,
		filter_DefaultIsButton1		= 1,
		filter_CancelIsButton2		= 2,
		filter_DoesCursorTracking	= 4,	// not supported
		filter_System6Friendly		= 8,	// avoids Sys7 only calls
		
		filter_AlertDefaultBehaviour = (filter_DefaultIsButton1 | filter_CancelIsButton2)
		};
		
class UModalFilter {
public:
	static	ModalFilterUPP	GetFilter(
				EFilterBehaviour inBehaviour = filter_AlertDefaultBehaviour );
	
	static	void	SimulateButtonPress( DialogPtr inDialog, short inButton );
	static	void	OutlineButton( DialogPtr inDialog, short inButton );
	
protected:
	static	Boolean TestForButtonEquivalent( DialogPtr inDialog, 
					EventRecord *inEvent, Int16 inButton);
	
	static pascal Boolean sFilter(	DialogPtr 	 inCurrentDialog, 
									EventRecord *inEvent, 
									short 		*inDialogItem );
		
	static	ModalFilterUPP		sModalFilterUPP;
	static	EFilterBehaviour	sBehaviour;
};
