//
//	CQD3DErrorWindow.cp
//
//	class CQD3DErrorWindow
//	Send QuickDraw 3D errors to a TextEdit window.
//
//	I haven't had enough QD3D errors to really test this.
//
//	by James Jennings
//	November 23, 1995
//	Converted from an LSingleDoc to an LWindow, July 8, 1996
//

#include "CQD3DErrorWindow.h"
#include "QD3D Debug Macros.h"

#include <LTextEdit.h>
#include "CQD3DPane.h"

const PaneIDT	PaneID_TextPane = 9999;
const ResIDT	WIND_ErrorsDoc = 9997;

static const Int32	kMaxErrors = 25;	// maximum number of errors reported

// This is a "singleton" class: There can only be one of them, and that
// instance is stored in the class variable "singleton".
CQD3DErrorWindow *CQD3DErrorWindow::sSingleton = 0;

CQD3DErrorWindow * CQD3DErrorWindow::CreateFromStream( LStream *inStream )
{
	return new CQD3DErrorWindow( inStream );
}

void CQD3DErrorWindow::Install( LCommander *inSuper )
{
	ThrowIf_(sSingleton!=0);	// we can't make more than one of these
	URegistrar::RegisterClass(class_ID, (ClassCreatorFunc) CreateFromStream);
	
	LWindow::CreateWindow(WIND_ErrorsDoc, inSuper);
}

CQD3DErrorWindow::CQD3DErrorWindow( LStream *inStream )
	: LWindow( inStream ), mTE(0), mNumberOfErrors(0)
{
	ThrowIf_(sSingleton!=0);	// we can't make more than one of these
	
	// register our error handlers
	TQ3Status status;
	status = ::Q3Error_Register(ErrorMethod, 0);
	ThrowIfQ3Fail_(status);
	status = ::Q3Warning_Register(WarningMethod, 0);
	ThrowIfQ3Fail_(status);
	status = ::Q3Notice_Register(NoticeMethod, 0);
	ThrowIfQ3Fail_(status);
	
	// make sure we can find this later
	sSingleton = this;
}

CQD3DErrorWindow::~CQD3DErrorWindow()
{
	// Is this how you remove an error handler?
	TQ3Status status;
	status = ::Q3Error_Register( nil, 0 );
	ThrowIfQ3Fail_(status);
	status = ::Q3Warning_Register( nil, 0 );
	ThrowIfQ3Fail_(status);
	status = ::Q3Notice_Register( nil, 0 );
	ThrowIfQ3Fail_(status);
}

void CQD3DErrorWindow::FinishCreateSelf()
{
	// Fetch the TERecord so we can easily find it later.
	
	LTextEdit *pane = (LTextEdit*)FindPaneByID(PaneID_TextPane);
	ThrowIfNil_ (pane);
	
	// Remember the TE pane that we'll be writing to.
	mTE = pane->GetMacTEH();
	ThrowIfNil_ (mTE);
	
}

//
//	Error handling functions. [static]
//
void 
CQD3DErrorWindow::ErrorMethod(  TQ3Error first, TQ3Error last, long reference)
{
	TQ3Error dummy;
	OSErr fst, lst;
	CQD3DErrorWindow *theDoc = CQD3DErrorWindow::GetInstance();
	if (theDoc) {
		if (Q3Error_IsFatalError(first)) {
			
			CQD3DPane::FatalErrorSeen();
			
			theDoc->Common("\pFATAL ERROR", (long)first, (long)last);
			if (first==kQ3ErrorMacintoshError) {
				fst = ::Q3MacintoshError_Get(&lst);
				theDoc->Common("\pFatal Macintosh Error was", (long)fst, (long)lst);
			}
		} else {
			theDoc->Common("\pError", (long)first, (long)last);
			if (first==kQ3ErrorMacintoshError) {
				fst = ::Q3MacintoshError_Get(&lst);
				theDoc->Common("\pMacintosh Error was", (long)fst, (long)lst);
			}
		}
		/* This clears the sticky error */
		::Q3Error_Get(&dummy);
	}
}

void 
CQD3DErrorWindow::WarningMethod(TQ3Warning first, TQ3Warning last, long reference)
{
	CQD3DErrorWindow *theDoc = CQD3DErrorWindow::GetInstance();
	if (theDoc) {
		theDoc->Common("\pWarning", (long)first, (long)last);
	}
}

void 
CQD3DErrorWindow::NoticeMethod( TQ3Notice first, TQ3Notice last, long reference)
{
	CQD3DErrorWindow *theDoc = CQD3DErrorWindow::GetInstance();
	if (theDoc) {
		theDoc->Common("\pNotice", (long)first, (long)last);
	}
}

//
// Common error handling function
//
	static void	// helper function
	DoErrorText(TEHandle hTE, StringPtr type, long err)
	{
		const Str63 del = "\p: ";	// standard delimeter
		LStr255 theText(type);
		theText += del;	// write delimeter
		theText += TString<Str15>(err);	// write error number (the cast converts the int)
		// Get error string and insert it here
		if (short(err) == err) {
			// it's a short error: look up the name in a resource
			theText += del;	// write delimeter
			StResource str('Estr', err, false); // don't throw if failed
			if (str.mResourceH) {
				StHandleLocker strLock(str);
				theText += '';
				theText += (StringPtr)*(str.mResourceH);
				theText += '';
			} else {
				theText += "\p(no error description available)";
			}
		} else {
			// it's a long error: Do nothing.
		//	theText.Append(&err, 4);
		}
		
		// Stuff the text into the TEHandle
		::TEInsert(ConstStringPtr(theText)+1, theText.Length(), hTE);
		
	}
	// end helper function
void
CQD3DErrorWindow::Common(StringPtr type, long first, long last)
{
	// limit the number of errors we will report.
	mNumberOfErrors++;
	if (mNumberOfErrors > kMaxErrors) return;
	
	Show();	// window is hidden until the first error
	
	// move insertion to the end
	Int16 teLength = (**mTE).teLength;
	::TESetSelect(teLength, teLength, mTE);	
	
	DoErrorText(mTE, type, first);
	
	if (last != 0) { 
	// if it's not kQ3ErrorNone, kQ3WarningeNone, or kQ3NoticeNone
	//	const Str63 lst = "\p: Last ";
	//	::TEInsert(lst+1, *lst, mTE);	// write "Last "
		Write( "\p\r  Last " );
		DoErrorText(mTE, type, last);
	}
	TEKey('\r', mTE);	// write newline
	
	if (mNumberOfErrors >= kMaxErrors) {
	//	const Str63 lst = "\p: ***** Too Many Errors *****\r";
	//	::TEInsert(lst+1, *lst, mTE);	// write "Last "
		Write( "\p: ***** Too Many Errors *****\r" );
	}
	
	return;
}

void	CQD3DErrorWindow::Write( ConstStringPtr str )
{
	::TEInsert(str+1, *str, mTE);
}
